
import os
import sys
import socket
import logging
from logging.handlers import SysLogHandler

class LocalSyslogHandler(SysLogHandler):
    def emit(self, record):
        '''
        Takes a log message and breaks it out into multiple messages if it exceeds 64k for syslogger.
        Overload function since the original will throw an error if message is over the limit.
        '''
        maxlen = 2015 if sys.platform == 'darwin' else 65502
        mesg = record.getMessage()
        if len(mesg) > maxlen:
            while len(mesg) > maxlen:
                record.msg = mesg[0:maxlen]
                super().emit(record)
                mesg = mesg[maxlen:]
        else:
            super().emit(record)

def getLogger(name, log_level=None, log_format='standard'):
    '''
    Get a logger by the name provided. Set the log_level if you provide the second argument.
    If not, ask the environment for $LOG_LEVEL. If not, default to DEBUG.
    Also attaches a syslog handler that will handle large strings and stream the message
    to the syslog endpoint.
    @param log_level {enum:string} One of the logger levels.
    @return {@logging.getLogger()}
    '''
    log_levels = ['DEBUG', 'INFO', 'WARN', 'WARNING', 'ERROR', 'FATAL', 'CRITICAL']
    log_level_map = dict([ (_ll, getattr(logging, _ll)) for _ll in log_levels ])
    if not log_level:
        #@throws KeyError if $LOG_LEVEL is set to invalid log level as described
        # by above array $log_level_map
        log_level = log_level_map[ os.getenv('LOG_LEVEL', 'DEBUG').upper() ]
    logFormats = {
      'standard': '%(asctime)s %(name)s.%(funcName)s(PID=%(process)d %(levelname)-8s) %(message)s',
      'json': '{ "time": "%(asctime)s", "function": "%(name)s.%(funcName)s", "pid": "%(process)d", "level": "%(levelname)s", "message", "%(message)s" }',
      'csv': '%(asctime)s,%(name)s.%(funcName)s,%(levelname)s,%(message)s',
    }
    format_str = logFormats.get(log_format, 'standard')
    logging.basicConfig(format=format_str, datefmt='%F %T')
    root = logging.getLogger()
    # AVoid duplicate log messages by removing root logger handlers.
    for h in root.handlers:
        root.removeHandler(h)

    formatter = logging.Formatter(format_str)
    if os.path.exists('/dev/log'):
        syslog_handler = LocalSyslogHandler(address=('/dev/log'), facility=SysLogHandler.LOG_DAEMON, socktype=socket.SOCK_STREAM)
    elif os.path.exists('/var/run/syslog'):
        syslog_handler = LocalSyslogHandler(address='/var/run/syslog', facility=SysLogHandler.LOG_DAEMON, socktype=socket.SOCK_STREAM)
    else: # Use UDP if we can't find a local syslog socket
        syslog_handler = LocalSyslogHandler(address=('localhost', 514), facility=SysLogHandler.LOG_DAEMON, socktype=socket.SOCK_DGRAM)
    syslog_handler.setLevel(log_level)
    syslog_handler.setFormatter(formatter)

    if sys.stderr.isatty():
        print_handler = logging.StreamHandler(sys.stderr)
        print_handler.setLevel(log_level)
        print_handler.setFormatter(formatter)

    logger = logging.getLogger(name)
    logger.setLevel(log_level)
    logger.addHandler(syslog_handler)
    if sys.stderr.isatty():
        logger.addHandler(print_handler)

    return logger




import copy
import yaml

def literal_scalar(self, tag, value, style=None):
    '''
    Maintains for us the discontinuity of a multiline scalar string.
    '''
    if style is None:
        if any(c in value for c in u"\u000a\u000d\u001c\u001d\u001e\u0085\u2028\u2029"):
            style='|'
        else:
            style = self.default_style
    node = yaml.representer.ScalarNode(tag, value, style=style)
    if self.alias_key is not None:
        self.represented_objects[self.alias_key] = node
    return node

def dump_yaml(data):
    '''
    Same as write yaml, but returns the result instead.
    Also use our representer to render multiline strings properly.
    '''
    yaml.representer.BaseRepresenter.represent_scalar = literal_scalar
    return yaml.dump(data, default_flow_style=False)

def read_yaml(f):
    '''
    Read a file and parse it as yaml.
    Return the data structure
    '''
    result = None
    with open(f, 'r') as fd:
        result = yaml.safe_load( fd.read() )
    return result

def write_yaml(f, data):
    '''
    Takes a data structure and writes it to file as yaml.
    Except, don't use the default options provided by the package.
    Use our custom decorators and whatnot to make it all purty <:
    '''
    with open(f, 'w') as fd:
        fd.write( dump_yaml(data) )
    return True

def dictmerge(target, *args):
    '''
    Merge multiple dictionaries.
    Usage:
    
        one = {'one': 1}
        two = {'two': 2}
        three = {'three': 3}
        sum = dictmerge(one, two, three)
        
        {one: 1, two: 2, three: 3}
    '''
    target = copy.deepcopy(target)
    if len(args) > 1:
        for obj in args:
            target = dictmerge(target, obj)
        return target
    obj = copy.deepcopy(args[0])

    # Try to combine basic algorithms we know will work for sure.
    try:
        if isinstance(target, (bool, int, float, str, list, tuple, set)) and isinstance(obj, (bool, int, float, str, list, tuple, set)):
            if isinstance(target, bool) and isinstance(obj, bool):
                return target and obj
            if isinstance(target, (int, float)) and isinstance(obj, (int, float)):
                return target + obj
            if isinstance(target, (list, tuple, set)) and isinstance(obj, (list, tuple, set)):
                return list(target + obj)
            raise ValueError('Invalid Argument')
        elif isinstance(target, dict) and isinstance(obj, dict):
            # Don't take action on dictionaries. We will execute on them in the following lines after the exception tracking.
            pass
        else:
            raise ValueError('Invalid Argument')
    except Exception as e:
        raise ValueError('\x1b[33mCould not merge\x1b[0m %s(%s) and %s(%s); Error=%s' % (type(target), target, type(obj), obj, e) )

    # Recursively merge dicts and set non-dict values
    for k, v in list(obj.items()):
        if k in target and isinstance(v, dict):
            target[k] = dictmerge(target[k], v)
        elif k in target and isinstance(v, (set, tuple, list)):
            oldtype = type(target[k])
            # Coerce to list so we can add the two reliably
            target[k] = oldtype( [*target[k]] + [*v] )
            # Convert back to its original type afterwards.
        else:
            target[k] = v
    return target





import os
from kizano.logger import getLogger
import kizano.utils as utils
log = getLogger(__name__)

class Config(object):
    '''
    Configuration object that tracks config in /etc/${APP_NAME}/config.yml and
    in ~/.config/${APP_NAME}/config.yml

    Returns a cached version of the config if we've asked the filesystem for it before for
    faster IO in subsequent reads.

    Reads system config in `/etc`, then overlays user config on top of that.
    Arrays are appended when encountered.
    '''

    APP_NAME = 'kizano'
    __CONFIGCACHE = {}

    @staticmethod
    def getConfig():
        """
        Static method for getting configuration for this app, be it cron, client or server.
        """
        if Config.__CONFIGCACHE:
            log.debug('Cache-HIT: Returning config from cache')
            return Config.__CONFIGCACHE
        cfgfiles = [
          os.path.join(os.path.sep, 'etc', Config.APP_NAME, 'config.yml'),
          os.path.join(os.environ['HOME'], '.config', Config.APP_NAME, 'config.yml')
        ]
        for cfgfile in cfgfiles:
            try:
                cfg = utils.read_yaml(cfgfile)
                log.info(f'Found and loaded {cfgfile}')
                Config.__CONFIGCACHE = utils.dictmerge(Config.__CONFIGCACHE, cfg)
            except Exception as e:
                log.info(f'Did not load {cfgfile}, reason={e}')
                continue
        log.debug(f'Cache-Miss: Config loaded - {Config.__CONFIGCACHE}')
        return Config.__CONFIGCACHE

getConfig = Config.getConfig




#!/usr/bin/env python3

import os
import sys
from glob import glob
from pprint import pprint
from setuptools import setup

sys.path.insert(0, os.path.abspath('lib'))

setup_opts = {
    'name'                : 'kizano',
    # We change this default each time we tag a release.
    'version'             : '1.0.3',
    'description'         : 'Kizano code for quick access to functions.',
    'long_description'    : ('This is a core library that should allow one to import and run with '
                            'pre-built libraries. Common features like logging and configuration '
                            'management should not need to be re-created in every project. This aims '
                            'to minimize this by providing a core library of swiss army knife of'
                            'common functionality.'),
    'long_description_content_type': 'text/markdown',
    'author'              : 'Markizano Draconus',
    'author_email'        : 'markizano@markizano.net',
    'url'                 : 'https://markizano.net/',
    'license'             : 'GNU',

    'tests_require'       : ['nose', 'mock', 'coverage'],
    'install_requires'    : [
      'PyYAML>=6.0.1',
    ],
    'package_dir'         : { 'kizano': 'lib/kizano' },
    'packages'            : [
      'kizano',
      'kizano.logger'
    ],
    'scripts'             : glob('bin/*'),
    'test_suite'          : 'tests',
}

try:
    import argparse
    HAS_ARGPARSE = True
except:
    HAS_ARGPARSE = False

if not HAS_ARGPARSE: setup_opts['install_requires'].append('argparse')

# I botch this too many times.
if sys.argv[1] == 'test':
    sys.argv[1] = 'nosetests'

if 'DEBUG' in os.environ: pprint(setup_opts)

setup(**setup_opts)
